#' Construct from the data a base for the pseudo-random number seeds to be used in the final protection. 
#' 
#' @param .g list of temporary objects needed.
#' @return Updated version of .g.
.jk.make.seed.base <- function(.g){
	# requires:
	#	group.by.ids
	#	group.by.ids.sql
	#	grouped.table
	#	mysql.connection
	#	replacements.sql1
	#	replacements.sql2
	#	table.with.group.ids
	#	where
    #   .jk.secret
	# reads:
	#	_level_combinations
	# assigns:
	#	seed.base
	
	# Future work: 
	#	- use group.by columns also
	#	- use row and column ids instead of or in addition to values for hashing
	
	
	# handy abbreviations:
	
	x <- .g$analyse.names
	
		
	# hash the values to 32 hex digits:
	
	hashs.view <- cpaste(
	
		 " ( SELECT *, "
			,cpaste(
				" MD5(CONCAT('",.g$.jk.secret,"',",x,")) AS hash__",x
			)
		," FROM ",.g$table.with.group.ids," AS t ) "
	)

	# transform them to pairs of Uniform(0,16^8-1)-distributed integers:
	
	uniforms.view <- cpaste(
	
		 " ( SELECT *, "
			,cpaste(
				 "  CONV(SUBSTR(hash__",x,", 1,8),16,10) AS u1a__",x
				," ,CONV(SUBSTR(hash__",x,", 9,8),16,10) AS u1b__",x
				," ,CONV(SUBSTR(hash__",x,",17,8),16,10) AS u2a__",x
				," ,CONV(SUBSTR(hash__",x,",25,8),16,10) AS u2b__",x
			)
		," FROM ",hashs.view," AS h ) "
	)

	# transform them to Normal(0,1)-distributed using Box-Muller:
	
	normals.view <- cpaste(
	
		 " ( SELECT *, "
			,cpaste(
				 "  SQRT(22.180709777918-LOG(u1a__",x,"))*COS(u1b__",x,") AS n1__",x
				," ,SQRT(22.180709777918-LOG(u2a__",x,"))*COS(u2b__",x,") AS n2__",x
			)
		," FROM ",uniforms.view," AS u ) "
	)
	
	# transform them to replacements using prepared code:
	
	replacements.view <- cpaste(
	
		" ( SELECT *, "
			,cpaste(
				 .g$replacements.sql1," AS r1__",x,", "
				,.g$replacements.sql2," AS r2__",x
			)
		," FROM ",normals.view," AS n ) "			
	)
	

	# check the where-condition on the true and the replacement values,
	# and construct a group-hash from those for which the where-condition is true:
	
	where.r1 <- lapply(x,function(xi){replace.word(xi,cpaste("r1__",xi),.g$where)})
	where.r2 <- lapply(x,function(xi){replace.word(xi,cpaste("r2__",xi),.g$where)})
	
	.g$seed.base <- 
		cpaste(
	
		 " ( SELECT COUNT(*) AS n__fullgroup, "
			,cpaste(
				" SUBSTR(full_",.g$group.by.ids,",1,len_",.g$group.by.ids,") AS ",.g$group.by.ids
			),","
			,cpaste(
				"  SUM(  CASE WHEN ",.g$where," THEN u1a__",x,"-",16^8/2," ELSE 0 END "
					," + CASE WHEN ",where.r1," THEN u1b__",x,"-",16^8/2," ELSE 0 END "
					," + CASE WHEN ",where.r2," THEN u2b__",x,"-",16^8/2," ELSE 0 END ) AS sb__",x
			)
		," FROM ",replacements.view," AS r, _level_combinations AS l "
		," WHERE "
			,paste(
				 " LENGTH(full_",.g$group.by.ids,")>=len_",.g$group.by.ids
				,sep="",collapse=" AND "
			)
		," GROUP BY ",.g$group.by.ids.sql
		," ) "
	)
	
	if (.g$.jk.debug) {
		print("hashs.view")
		print(hashs.view)
		#print(dbGetQuery(.g$mysql.connection,hashs.view))
		print("uniforms.view")
		print(uniforms.view)
		#print(dbGetQuery(.g$mysql.connection,uniforms.view))
		print("normals.view")
		print(normals.view)
		#print(dbGetQuery(.g$mysql.connection,normals.view))
		print("replacements.view")
		print(replacements.view)
		print(dbGetQuery(.g$mysql.connection,replacements.view))
		print(dbGetQuery(.g$mysql.connection,cpaste(
			"SELECT "
				,cpaste(
					 "MIN(r1__",.g$analyse.names,") AS min_r1__",.g$analyse.names,","
					,"AVG(r1__",.g$analyse.names,") AS mean_r1__",.g$analyse.names,","
					,"MAX(r1__",.g$analyse.names,") AS max_r1__",.g$analyse.names,","
					,"STD(r1__",.g$analyse.names,") AS std_r1__",.g$analyse.names
				)
			," FROM ",replacements.view," AS r")))
		print("where.r1")
		print(where.r1)
		print("where.r2")
		print(where.r2)
		print("seed.base")
		print(.g$seed.base)
		print(dbGetQuery(.g$mysql.connection,.g$seed.base))
	}
	return(.g)
}
